#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <ctype.h>

#define MAXLINE 1000

char *fullpath(char *fname, char *fpath);

int cdecl main(int argv, char *argc[])
{
	FILE *a_file;
	struct find_t found;
	int i;
	int numfiles;
	unsigned int nofile;
	unsigned int linenum;
	unsigned char returnval;
	unsigned char pflag;
	char inline[MAXLINE];
	char fname[_MAX_DIR];
	char fpath[_MAX_PATH];
	char *ipos;
	char *apos;


	/************************************************************************/
	/*                                                                      */
	/*   Exit if improper syntax was used.                                  */

	if (argv < 3) {
		fprintf(stderr, "FINDIT v1.2 by Peter Nelson, 8-17-88: Invalid argument.  Usage:\n");
		fprintf(stderr, "findit <wildcard> \"text to locate\" \"text\" \"text\" ...\n");
		returnval = 3;
		goto main_exit;
	}


	/************************************************************************/
	/*                                                                      */
	/*   Determine the full pathname                                        */

	strcpy(fname, argc[1]);
	fullpath(fname, fpath);
	
	
	/************************************************************************/
	/*                                                                      */
	/*   Convert all the text strings to lower case                         */
	
	for (i=2; i<argv; i++)
		strlwr(argc[i]);


	/************************************************************************/
	/*                                                                      */
	/*   Loop as long as we find matching filenames                         */

	numfiles = 0;
	nofile = _dos_findfirst(fname, _A_NORMAL, &found);
	while (!nofile) {

		numfiles++;
		strcpy(fname, fpath);
		strcat(fname, found.name);


		/*********************************************************************/
		/*                                                                   */
		/*   Reset the line new-file flag and the line number counter        */
	
		pflag = 0;
		linenum = 0;

		/*********************************************************************/
		/*                                                                   */
		/*   Open the file                                                   */
	
		if (!(a_file = fopen(fname, "r"))) goto openfile_error;

		/*********************************************************************/
		/*                                                                   */
		/*   Loop as long as there are lines in the file                     */

		while (!feof(a_file)) {

			/******************************************************************/
			/*                                                                */
			/*   Read the next line in the file.  Increment the line number   */
			/*   counter, and convert the line to lower case.                 */

			fgets(inline, MAXLINE, a_file);
			if (ferror(a_file)) goto readfile_error;

			if (!feof(a_file)) {
			
				linenum++;

				/***************************************************************/
				/*                                                             */
				/*   Check if the line contains any of the text entered.  And  */
				/*   print them if they do.                                    */
	
				for (i=2; i<argv; i++) {

					/* find a match, ignore upper/lower case */
					for (ipos = inline, apos = argc[i]; *ipos && *apos; ipos++)
						if (*apos++ != tolower(*ipos))
							apos = argc[i];

					if (!*apos) {		/* print if there was a match */

						/* remove trailing garbage */
						while (*ipos)
							ipos++;
						while (!isgraph(*ipos) && ipos >= inline)
							*ipos-- = '\0';

						if (!pflag) {
							pflag = 1;
							fprintf(stdout, "\n\n%s\n(%05u): %s\n", fname, linenum, inline);
							break;
						}
						else {
							fprintf(stdout, "(%05u): %s\n", linenum, inline);
							break;
						}
					}
				}
			}
		}
	
		/*********************************************************************/
		/*                                                                   */
		/*   Close the file.                                                 */

		if (fclose(a_file)) goto closefile_error;
		

		/*********************************************************************/
		/*                                                                   */
		/*   Find the next matching filename                                 */

		nofile = _dos_findnext(&found);	
	}

	if (!numfiles) goto nofile_error;

	returnval = 0;
	goto main_exit;


openfile_error:
	fprintf(stderr, "Error opening file %s:  %s\n", fname, sys_errlist[errno]);
	returnval = 1;
	goto main_exit;


readfile_error:
	fprintf(stderr, "Error reading file %s:  %s\n", fname, sys_errlist[errno]);
	returnval = 1;
	goto main_exit;


closefile_error:
	fprintf(stderr, "Error closing file %s:  %s\n", fname, sys_errlist[errno]);
	returnval = 1;
	goto main_exit;


nofile_error:
	fprintf(stderr, "File \"%s\" not found.\n", fname);
	returnval = 2;
	goto main_exit;


main_exit:
	return(returnval);

}


/***************************************************************************/
/*                                                                         */
/*   fullpath()    - Covert a partial path into a full path                */
/*                                                                         */
/*   char *fullpath(char *fname, char *fpath);                             */
/*                                                                         */
/*   This routine accepts as input, a string of characters pointed to by   */
/*   *fname representing a filename.  Example:  C:MYDIR\MYFILE  The        */
/*   routine will determine what is missing from the path (in the example  */
/*   above, the root directory, and the extension are missing) and         */
/*   supplies them using the current system defaults.  The string *fname   */
/*   is then converted into the full path name and filename (in the        */
/*   example above, C:\MYDIR\MYFILE.*).  The string *fpath is equivalent   */
/*   to *fname, but without the filename specification (in the example     */
/*   above, C:\MYDIR\).  This is meant to be used in conjunction with the  */
/*   DOS findfirst/findnext wildcard processing functions, where only      */
/*   the file name is returned, and not the path name.  The routine        */
/*   returns the pointer *fname.                                           */
/*                                                                         */
/***************************************************************************/

#include <direct.h>

char *fullpath(char *fname, char *fpath)
{
	unsigned int n;
	unsigned int d;

	char fdir[_MAX_DIR];
	char xfname[_MAX_FNAME];
	char xdrive[_MAX_DRIVE];
	char xdir[_MAX_DIR];
	char xfext[_MAX_EXT];

	
	/************************************************************************/
	/*                                                                      */
	/*   Determine the full pathname/filename of the wildcard to scan       */

	/* first parse up the wildcard they entered as the first parameter */
	/* on the command line */
	_splitpath(fname, xdrive, xdir, xfname, xfext);
	strupr(xdrive);
	strupr(xdir);
	strupr(xfname);
	strupr(xfext);

	/* this is to deal with a bug in the splitpath command which considers */
	/* the  ..  in   engine ..  to be the filename and not the path name */
	/*
	if (!strcmp(xfname, ".") && !strcmp(xfext, ".")) {
		strcat(xdir, "..\\");
		xfname[0] = '\0';
		xfext[0] = '\0';
	}
	*/

	/* get the default drive */
	_dos_getdrive(&d);

	/* did they designate a drive? */
	if (!xdrive[0]) {
		/* if not, convert the current drive number into a letter */
		xdrive[0] = (char) d - 1 + 'A';
		xdrive[1] = ':';
		xdrive[2] = '\0';
	}

	/* did they enter a directory path which could be expanded upon? */
	if (xdir[0] == '\\') {
		/* if so, copy the drive and directory into the full path name */
		strcpy(fpath, xdrive);
		strcat(fpath, xdir);
	}

	else {
		
		/* if they entered a path which might be expanded upon, look up */
		/* the current default directory on the drive they specified.  Then */
		/* copy the drive and the default directory and the directory they */
		/* named into the full path name */
		
		_dos_setdrive((int) xdrive[0] + 1 - 'A', &n);		

		if (!getcwd(fdir, _MAX_DIR)) {
			fprintf(stderr, "Error reading drive %c:  %s\n", xdrive[0], sys_errlist[errno]);
			_dos_setdrive(d, &n);
			exit(1);
		}

		if (errno) {
			fprintf(stderr, "Error reading drive %c:  %s\n", xdrive[0], sys_errlist[errno]);
			_dos_setdrive(d, &n);
			exit(1);
		}
		
		_dos_setdrive(d, &n);

		strcpy(fpath, fdir);
		if (*(fdir+strlen(fdir)-1) != '\\')
			strcat(fpath, "\\");
		strcat(fpath, xdir);
	}

	/* see if the filename they entered was actually the name of a */
	/* directory.  If so, tack *.* onto the end of it. */

	strcpy(fname, fpath);
	strcat(fname, xfname);
	strcat(fname, xfext);

	if (!getcwd(fdir, _MAX_DIR)) {
		fprintf(stderr, "Error reading drive %c:  %s\n", xdrive[0], sys_errlist[errno]);
		exit(1);
	}

	if (errno) {
		fprintf(stderr, "Error reading drive %c:  %s\n", xdrive[0], sys_errlist[errno]);
		exit(1);
	}
	
	/* try to change to that directory */

	if (!chdir(fname)) {
		/* they asked for a directory, so add a \*.* to it */
		strcat(fpath, xfname);
		strcat(fpath, xfext);
		strcat(fpath, "\\");
		xfname[0] = '\0';
		xfext[0] = '\0';
		/* and change back to the original directory */
		chdir(fdir);
	}

	/* if they didn't enter a filename, make it an asterisk so it */
	/* will default to all files */
	if (!xfname[0])
		strcpy(xfname, "*");

	/* same goes for the extension */
	if (!xfext[0])
		strcpy(xfext, ".*");
	
	/* copy the full file name and the extension they entered into the */
	/* full file name */

	strcpy(fname, fpath);
	strcat(fname, xfname);
	strcat(fname, xfext);

	/* return a pointer to the full path name */
	return(fname);
}
